package com.topcoder.db.generation.xsd2ddl;

import java.io.IOException;
import java.io.InputStream;

import com.topcoder.db.generation.xsd2ddl.data.Database;

/**
 * This interface abstracts a database definition parser (the parsing layer of the database
 * definition transformation process).
 * 
 * The source of the definition is an InputStream. The content of the stream could be
 * anything, depending on the implementation.
 * 
 * The parsing process produces a Database data class that contains instances of all the
 * other classes from the data package (Field, FieldType, Table, TableType, Key).
 * 
 * Essentially what this interface does is converting the database definition from an
 * implementation specific format into a generic, implementation independent format
 * represented by the classes in the data package. 
 * 
 * Note:
 * 
 * All implementations are expected to have a public constructor taking a configuration
 * namespace (String) as parameter. This constructor is invoked by the main class using reflection.
 * This allows configuring the class to be used in a configuration file and it allows
 * that class itself to read configuration information.
 * 
 * Thread safety:
 * 
 * The implementations are typically not thread safe because they will most likely
 * need to keep some state during parsing. It is expected multiple parse calls
 * to be invoked sequentially, not concurrently.
 * 
 * @author TCSDESIGNER
 */
public interface DefinitionParser {
	
	/**
	 * Parses the database definition from the input stream and creates a Database
	 * structure (including Field, FileType, Key, Table, TableType instances).
	 * 
	 * The format of the input is specific to the implementation.
	 * 
	 * @param definitionStream the stream that contains the definition
	 * @return the parsed database definition
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws IOException if an I/O error occurs while working with the stream
	 * @throws ParseException if a parser implementation specific error occurs
	 */
	public Database parse(InputStream definitionStream) throws IllegalArgumentException, IOException, ParseException;

}
