package com.topcoder.db.generation.xsd2ddl;

import java.io.IOException;
import java.io.OutputStream;

import com.topcoder.db.generation.xsd2ddl.data.Database;

/**
 * This interface abstracts a database definition rendering (the rendering layer of the database
 * definition transformation process).
 *
 * The rendering process takes as input a Database object that contains instances of all the
 * other classes from the data package (Field, FieldType, Table, TableType, Key).
 *
 * The destination of the rendering in an output stream. The content that is written to the output
 * stream could be anything, depending on the implementation (but will generally consist of SQL statements).
 * 
 * Essentially what this interface does is converting the database definition 
 * from an implementation independent format represented by the classes in the data package
 * to an implementation specific format (generally SQL statements suitable for some specific database). 
 * 
 * Note:
 * 
 * All implementations are expected to have a public constructor taking a configuration
 * namespace (String) as parameter. This constructor is invoked by the main class using reflection.
 * This allows configuring the class to be used in a configuration file and it allows
 * that class itself to read configuration information.
 * 
 * Thread safety:
 * 
 * The implementations are typically thread safe because they will most likely
 * not need to keep state during rendering. An implementation is likely to handle
 * concurrent render calls in a thread safe manner (but it is not a requirement).
 * 
 * @author TCSDESIGNER
 */
public interface DefinitionRenderer {
	
	/**
	 * Renders the database definition from input Database object (can include
	 * Field, FileType, Key, Table, TableType instances) to a stream
	 * 
	 * The format of the output is specific to the implementation.
	 * 
	 * @param definition the database definition
	 * @param renderStream the destination stream
	 * 
	 * @throws IllegalArgumentException if any argument is null
	 * @throws IOException if an I/O error occurs while working with the stream
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void render(Database definition, OutputStream renderStream) throws IllegalArgumentException, IOException, RenderException;

}
