package com.topcoder.db.generation.xsd2ddl;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.SQLException;

import com.topcoder.util.config.ConfigManagerException;

/**
 * This is the main class of the design. This class essentially covers all the functionality
 * a user is exposed to in the normal usage scenarios (facade pattern).
 * 
 * What this class does is essentially coordinating the three layers of this component
 * (parsing, type mapping and rendering) in order to perform the database definition
 * transformation.
 * 
 * The class contains the parser and renderer instances (that can be given either explicitely
 * or dynamically instantiated from a configuration namespace).
 * 
 * There is a big number of overloads allowing different combinations of source and target
 * types (String content, streams, files).
 * 
 * There are also some interesting overloads that given a JDBC connection will actually
 * run the produced output script against the database.
 * 
 * Finally, the class has a static main method that exposes a command line interface
 * that has a very rich collection of switches that control the transformation process.  
 * 
 * Thread safety:
 * 
 * This class is thread safe by its own because it is not mutable but the classes it uses are
 * not necessarily thread safe (DefinitionParser and DefinitionRenderer implementations).
 * Therefore, the thread safety depends on the thread safety of this classes
 * 
 * @author TCSDESIGNER
 */
public class DefinitionUtil {
	
	/**
	 * Constant for the default namespace.
	 * Public so that if some other component needs to specify the namespace used by this
	 * component, it doesn't have to hardcode it.
	 */
	public static final String DEFAULT_NAMESPACE = "com.topcoder.db.generation.xsd2ddl";
	
	/**
	 * Stores the namespace that is used to instantiate this class.
	 * Cannot be null, empty or all spaces.
	 * Set in the constructors either to a given value or to DEFAULT_NAMESPACE.
	 * Used by the execute methods. 
	 */
	private String namespace = null;
	
	/**
	 * The definition parser instance.
	 * Cannot be null.
	 * Set in the constructors either to a given value or to the value defined in the configuration namespace.
	 * Used by all the produce and execute methods and its getter. 
	 */
	private DefinitionParser definitionParser = null;

	/**
	 * The definition renderer instance.
	 * Cannot be null.
	 * Set in the constructors either to a given value or to the value defined in the configuration namespace.
	 * Used by all the produce and execute methods and its getter. 
	 */
	private DefinitionRenderer definitionRenderer = null;
	
	/**
	 * Constructor that uses the default namespace for configuring the instance.
	 * 
	 * Implementation notes:
	 * 
	 * Call DefinitionUtil(DEFAULT_NAMESPACE).
	 * 
	 * @throws ConfigManagerException if a configuration reading error occurs while reading from the namespace,
	 * if a required property is missing or if a reflection error occurs
	 */
	public DefinitionUtil() throws ConfigManagerException {
	}
	
	/**
	 * Constructor that uses a given namespace for configuring the instance.
	 * 
	 * Implementation notes:
	 * 
	 * - read the parser_class and renderer_class properties 
	 * - find the constructors taking the namespace (String) as argument
	 * - invoke the constructors, passing the namespace as argument
	 * - set the corresponding attributes with the obtained instances
	 *
	 * @param namespace the namespace to load the configuration from
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 * @throws ConfigManagerException if a configuration reading error occurs while reading from the namespace,
	 * if a required property is missing or if a reflection error occurs
	 */
	public DefinitionUtil(String namespace) throws IllegalArgumentException, ConfigManagerException {
	}
	
	/**
	 * Constructor that uses the default namespace for configuring the instance but
	 * also allows overriding any of the parser or renderer instances.
	 * 
	 * Any null value will cause the corresponding attribute to be loaded from
	 * the configuration namespace (but any non-null argument takes precedence).
	 * 
	 * Implementation notes:
	 * 
	 * Call DefinitionUtil(DEFAULT_NAMESPACE, definitionParser, definitionRenderer)
	 *
	 * @param definitionParser the definition parser to use (can be null)
	 * @param definitionRenderer the definition renderer to use (can be null)
	 * 
	 * @throws ConfigManagerException if a configuration reading error occurs while reading from the namespace,
	 * if a required property is missing or if a reflection error occurs
	 */
	public DefinitionUtil(DefinitionParser definitionParser, DefinitionRenderer definitionRenderer) throws ConfigManagerException {
	}

	/**
	 * Constructor that uses a given namespace for configuring the instance but
	 * also allows overriding any of the parser or renderer instances.
	 * 
	 * Any null value will cause the corresponding attribute to be loaded from
	 * the configuration namespace (but any non-null argument takes precedence).
	 * 
	 * Implementation notes:
	 * 
 	 * - if definitionParser or definitionRenderer are null, read the parser_class or 
 	 *   renderer_class properties 
	 * - find the constructors taking the namespace (String) as argument
	 * - invoke the constructors, passing the namespace as argument
	 * - set the corresponding attributes with the obtained instances
	 *
	 * @param namespace the namespace to load the configuration from
	 * @param definitionParser the definition parser to use (can be null)
	 * @param definitionRenderer the definition renderer to use (can be null)
	 * 
	 * @throws IllegalArgumentException if the namespace is null, empty or all spaces
	 * @throws ConfigManagerException if a configuration reading error occurs while reading from the namespace,
	 * if a required property is missing or if a reflection error occurs
	 */
	public DefinitionUtil(String namespace, DefinitionParser definitionParser, DefinitionRenderer definitionRenderer) throws IllegalArgumentException, ConfigManagerException {
	}
	
	/**
	 * Getter for the definition parser (definitionParser attribute).
	 * 
	 * @return the definition parser
	 */
	public DefinitionParser getDefinitionParser() {
		return definitionParser;
	}

	/**
	 * Getter for the definition renderer (definitionRenderer attribute).
	 * 
	 * @return the definition renderer
	 */
	public DefinitionRenderer getDefinitionRenderer() {
		return definitionRenderer;
	}

	/**
	 * Transforms the database definition read from a string source to a string destination.
	 * 
	 * The transformation consists of 2 phases: the parsing of the definitions from the source
	 * and the rendering of the definitions to the destination. The format of the source input depends
	 * on the concrete DefinitionParser instance that is used (could be XSD for example).
	 * The format of the destination output depends on the concrete DefinitionRenderer instance that
	 * is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * Call produceDefinitions(new StringBufferInputStream(new StringBuffer(definitionContent))).
	 *   
	 * @param definitionContent the source definition content
	 * @return the destination definition content
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public String produceDefinitions(String definitionContent) throws IllegalArgumentException, ParseException, TypeMappingException, RenderException {
		return null;
	}

	/**
	 * Transforms the database definition read from a stream source to a string destination.
	 * 
	 * The transformation consists of 2 phases: the parsing of the definitions from the source
	 * and the rendering of the definitions to the destination. The format of the source input depends
	 * on the concrete DefinitionParser instance that is used (could be XSD for example).
	 * The format of the destination output depends on the concrete DefinitionRenderer instance that
	 * is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * - baos = new ByteArrayOutputStream()
	 * - produceDefinitions(definitionStream, baos)
	 * - return new String(baos.getBytes())
	 *   
	 * @param definitionStream the stream that contains the definition
	 * @return the destination definition content
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws IOException if an I/O error occurs while working with the stream
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public String produceDefinitions(InputStream definitionStream) throws IllegalArgumentException, IOException, ParseException, TypeMappingException, RenderException {
		return null;
	}
	
	/**
	 * Transforms the database definition read from a file source to a string destination.
	 * 
	 * The transformation consists of 2 phases: the parsing of the definitions from the source
	 * and the rendering of the definitions to the destination. The format of the source input depends
	 * on the concrete DefinitionParser instance that is used (could be XSD for example).
	 * The format of the destination output depends on the concrete DefinitionRenderer instance that
	 * is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * Call produceDefinitions(new FileInputStream(definitionFile))
	 *   
	 * @param definitionFile the file that contains the definition
	 * @return the destination definition content
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 * @throws IOException if an I/O error occurs while working with the file
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public String produceDefinitionsFromFile(String definitionFile) throws IllegalArgumentException, IOException, ParseException, TypeMappingException, RenderException {
		return null;
	}

	/**
	 * Transforms the database definition read from a string source to a stream destination.
	 * 
	 * The transformation consists of 2 phases: the parsing of the definitions from the source
	 * and the rendering of the definitions to the destination. The format of the source input depends
	 * on the concrete DefinitionParser instance that is used (could be XSD for example).
	 * The format of the destination output depends on the concrete DefinitionRenderer instance that
	 * is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * Call produceDefinitions(new StringBufferInputStream(new StringBuffer(definitionContent)), renderStream).
	 *   
	 * @param definitionContent the source definition content
	 * @param renderStream the destination stream
	 * 
	 * @throws IllegalArgumentException if any argument is null, the string is empty or all spaces
	 * @throws IOException if an I/O error occurs while working with the stream
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void produceDefinitions(String definitionContent, OutputStream renderStream) throws IllegalArgumentException, IOException, ParseException, TypeMappingException, RenderException {
	}

	/**
	 * Transforms the database definition read from a stream source to a stream destination.
	 * 
	 * The transformation consists of 2 phases: the parsing of the definitions from the source
	 * and the rendering of the definitions to the destination. The format of the source input depends
	 * on the concrete DefinitionParser instance that is used (could be XSD for example).
	 * The format of the destination output depends on the concrete DefinitionRenderer instance that
	 * is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * - db = definitionParser.parse(definitionStream)
	 * - definitionRenderer.render(db, renderStream)
	 *   
	 * @param definitionStream the stream that contains the definition
	 * @param renderStream the destination stream
	 * 
	 * @throws IllegalArgumentException if any argument is null
	 * @throws IOException if an I/O error occurs while working with the streams
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void produceDefinitions(InputStream definitionStream, OutputStream renderStream) throws IllegalArgumentException, IOException, ParseException, TypeMappingException, RenderException {
	}
	
	/**
	 * Transforms the database definition read from a file source to a stream destination.
	 * 
	 * The transformation consists of 2 phases: the parsing of the definitions from the source
	 * and the rendering of the definitions to the destination. The format of the source input depends
	 * on the concrete DefinitionParser instance that is used (could be XSD for example).
	 * The format of the destination output depends on the concrete DefinitionRenderer instance that
	 * is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * Call produceDefinitions(new FileInputStream(definitionFile), renderStream)
	 *   
	 * @param definitionFile the file that contains the definition
	 * @param renderStream the destination stream
	 * 
	 * @throws IllegalArgumentException if any argument is null, the string is empty or all spaces
	 * @throws IOException if an I/O error occurs while working with the file or stream
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void produceDefinitionsFromFile(String definitionFile, OutputStream renderStream) throws IllegalArgumentException, IOException, ParseException, TypeMappingException, RenderException {
	}

	/**
	 * Transforms the database definition read from a string source to a file destination.
	 * 
	 * The transformation consists of 2 phases: the parsing of the definitions from the source
	 * and the rendering of the definitions to the destination. The format of the source input depends
	 * on the concrete DefinitionParser instance that is used (could be XSD for example).
	 * The format of the destination output depends on the concrete DefinitionRenderer instance that
	 * is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * Call produceDefinitions(definitionContent, new FileOutputStream(renderStream))
	 *   
	 * @param definitionContent the source definition content
	 * @param renderFile the destination file
	 * 
	 * @throws IllegalArgumentException if any argument is null, empty or all spaces
	 * @throws IOException if an I/O error occurs while working with the file
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void produceDefinitions(String definitionContent, String renderFile) throws IllegalArgumentException, IOException, ParseException, TypeMappingException, RenderException {
	}

	/**
	 * Transforms the database definition read from a stream source to a file destination.
	 * 
	 * The transformation consists of 2 phases: the parsing of the definitions from the source
	 * and the rendering of the definitions to the destination. The format of the source input depends
	 * on the concrete DefinitionParser instance that is used (could be XSD for example).
	 * The format of the destination output depends on the concrete DefinitionRenderer instance that
	 * is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * Call produceDefinitions(definitionStream, new FileOutputStream(renderStream))
	 *   
	 * @param definitionStream the stream that contains the definition
	 * @param renderFile the destination file
	 * 
	 * @throws IllegalArgumentException if any argument is null, the string is empty or all spaces
	 * @throws IOException if an I/O error occurs while working with the stream or file
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void produceDefinitions(InputStream definitionStream, String renderFile) throws IllegalArgumentException, IOException, ParseException, TypeMappingException, RenderException {
	}
	
	/**
	 * Transforms the database definition read from a file source to a file destination.
	 * 
	 * The transformation consists of 2 phases: the parsing of the definitions from the source
	 * and the rendering of the definitions to the destination. The format of the source input depends
	 * on the concrete DefinitionParser instance that is used (could be XSD for example).
	 * The format of the destination output depends on the concrete DefinitionRenderer instance that
	 * is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * Call produceDefinitionsFromFile(definitionFile, new FileOutputStream(renderStream))
	 *   
	 * @param definitionFile the file that contains the definition
	 * @param renderFile the destination file
	 * 
	 * @throws IllegalArgumentException if any argument is null, empty or all spaces
	 * @throws IOException if an I/O error occurs while working with the files
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */	
	public void produceDefinitionsFromFile(String definitionFile, String renderFile) throws IllegalArgumentException, IOException, ParseException, TypeMappingException, RenderException {
	}

	/**
	 * Transforms the database definition read from a string source and executes
	 * the obtained script against a database. The JDBC connection is configured in
	 * the configuration file and is obtained using DB Connection Factory.
	 * 
	 * The transformation consists of 3 phases: the parsing of the definitions from the source
	 * the rendering of the definitions to a string and the execution of the obtained output. 
	 * The format of the source input depends on the concrete DefinitionParser instance that is used
	 * (could be XSD for example). The format of the destination output depends on the concrete 
	 * DefinitionRenderer instance that is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * Call executeDefinitions(new StringBufferInputStream(new StringBuffer(definitionContent)))
	 *   
	 * @param definitionContent the source definition content
	 * 
	 * @throws IllegalArgumentException if any argument is null, empty or all spaces
	 * @throws SQLException if an SQL error occurs while executing the obtained definition script
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void executeDefinitions(String definitionContent) throws IllegalArgumentException, SQLException, ParseException, TypeMappingException, RenderException {
	}

	/**
	 * Transforms the database definition read from a stream source and executes
	 * the obtained script against a database. The JDBC connection is configured in
	 * the configuration file and is obtained using DB Connection Factory.
	 * 
	 * The transformation consists of 3 phases: the parsing of the definitions from the source
	 * the rendering of the definitions to a string and the execution of the obtained output. 
	 * The format of the source input depends on the concrete DefinitionParser instance that is used
	 * (could be XSD for example). The format of the destination output depends on the concrete 
	 * DefinitionRenderer instance that is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * - read the connection_name from the namespace
	 * - call executeDefinitions(definitionStream, connectionName)
	 *   
	 * @param definitionStream the stream that contains the definition
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws IOException if an I/O error occurs while working with the stream
	 * @throws SQLException if an SQL error occurs while executing the obtained definition script
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void executeDefinitions(InputStream definitionStream) throws IllegalArgumentException, IOException, SQLException, ParseException, TypeMappingException, RenderException {
	}
	
	/**
	 * Transforms the database definition read from a file source and executes
	 * the obtained script against a database. The JDBC connection is configured in
	 * the configuration file and is obtained using DB Connection Factory.
	 * 
	 * The transformation consists of 3 phases: the parsing of the definitions from the source
	 * the rendering of the definitions to a string and the execution of the obtained output. 
	 * The format of the source input depends on the concrete DefinitionParser instance that is used
	 * (could be XSD for example). The format of the destination output depends on the concrete 
	 * DefinitionRenderer instance that is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * Call executeDefinitions(new FileInputStream(definitionFile))
	 *   
	 * @param definitionFile the file that contains the definition
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 * @throws IOException if an I/O error occurs while working with the file
	 * @throws SQLException if an SQL error occurs while executing the obtained definition script
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void executeDefinitionsFromFile(String definitionFile) throws IllegalArgumentException, IOException, SQLException, ParseException, TypeMappingException, RenderException {
	}

	/**
	 * Transforms the database definition read from a string source and executes
	 * the obtained script against a database. The JDBC connection name is given as parameter
	 * and it is obtained using DB Connection Factory.
	 * 
	 * The transformation consists of 3 phases: the parsing of the definitions from the source
	 * the rendering of the definitions to a string and the execution of the obtained output. 
	 * The format of the source input depends on the concrete DefinitionParser instance that is used
	 * (could be XSD for example). The format of the destination output depends on the concrete 
	 * DefinitionRenderer instance that is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * Call executeDefinitions(new StringBufferInputStream(new StringBuffer(definitionContent)), connectionName)
	 *   
	 * @param definitionContent the source definition content
	 * @param connectionName the connection name to use
	 * 
	 * @throws IllegalArgumentException if any argument is null, empty or all spaces
	 * @throws SQLException if an SQL error occurs while executing the obtained definition script
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void executeDefinitions(String definitionContent, String connectionName) throws IllegalArgumentException, SQLException, ParseException, TypeMappingException, RenderException {
	}

	/**
	 * Transforms the database definition read from a stream source and executes
	 * the obtained script against a database. The JDBC connection name is given as parameter
	 * and it is obtained using DB Connection Factory.
	 * 
	 * The transformation consists of 3 phases: the parsing of the definitions from the source
	 * the rendering of the definitions to a string and the execution of the obtained output. 
	 * The format of the source input depends on the concrete DefinitionParser instance that is used
	 * (could be XSD for example). The format of the destination output depends on the concrete 
	 * DefinitionRenderer instance that is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * - read connection_factory_class and connection_namespace if present
	 * - create the connection factory class
	 * - obtain the connection with the given name
	 * - call executeDefinitions(definitionStream, connection)
	 *   
	 * @param definitionStream the source definition stream
	 * @param connectionName the connection name to use
	 * 
	 * @throws IllegalArgumentException if any argument is null, strings empty or all spaces
	 * @throws IOException if an I/O error occurs while working with the stream
	 * @throws SQLException if an SQL error occurs while executing the obtained definition script
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void executeDefinitions(InputStream definitionStream, String connectionName) throws IllegalArgumentException, IOException, SQLException, ParseException, TypeMappingException, RenderException {
	}
	
	/**
	 * Transforms the database definition read from a file source and executes
	 * the obtained script against a database. The JDBC connection name is given as parameter
	 * and it is obtained using DB Connection Factory.
	 * 
	 * The transformation consists of 3 phases: the parsing of the definitions from the source
	 * the rendering of the definitions to a string and the execution of the obtained output. 
	 * The format of the source input depends on the concrete DefinitionParser instance that is used
	 * (could be XSD for example). The format of the destination output depends on the concrete 
	 * DefinitionRenderer instance that is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * Call executeDefinitions(new FileInputStream(definitionFile), connectionName)
	 *   
	 * @param definitionFile the source definition file
	 * @param connectionName the connection name to use
	 * 
	 * @throws IllegalArgumentException if any argument is null, empty or all spaces
	 * @throws IOException if an I/O error occurs while working with the file
	 * @throws SQLException if an SQL error occurs while executing the obtained definition script
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void executeDefinitionsFromFile(String definitionFile, String connectionName) throws IllegalArgumentException, IOException, SQLException, ParseException, TypeMappingException, RenderException {
	}

	/**
	 * Transforms the database definition read from a string source and executes
	 * the obtained script against a database. The JDBC connection is given as parameter.
	 * 
	 * The transformation consists of 3 phases: the parsing of the definitions from the source
	 * the rendering of the definitions to a string and the execution of the obtained output. 
	 * The format of the source input depends on the concrete DefinitionParser instance that is used
	 * (could be XSD for example). The format of the destination output depends on the concrete 
	 * DefinitionRenderer instance that is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * Call executeDefinitions(new StringBufferInputStream(new StringBuffer(definitionContent)), connection)
	 *   
	 * @param definitionContent the source definition content
	 * @param connection the connection to use
	 * 
	 * @throws IllegalArgumentException if any argument is null, string empty or all spaces
	 * @throws SQLException if an SQL error occurs while executing the obtained definition script
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void executeDefinitions(String definitionContent, Connection connection) throws IllegalArgumentException, SQLException, ParseException, TypeMappingException, RenderException {
	}

	/**
	 * Transforms the database definition read from a stream source and executes
	 * the obtained script against a database. The JDBC connection is given as parameter.
	 * 
	 * The transformation consists of 3 phases: the parsing of the definitions from the source
	 * the rendering of the definitions to a string and the execution of the obtained output. 
	 * The format of the source input depends on the concrete DefinitionParser instance that is used
	 * (could be XSD for example). The format of the destination output depends on the concrete 
	 * DefinitionRenderer instance that is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * - script = produceDefinitions(definitionStream)
	 * - split script around ";\r\n" (; at end of line)
	 * - create Statement from Connection
	 * - for each split statement, execute it using the Statement
	 *    
	 * @param definitionStream the source definition stream
	 * @param connection the connection to use
	 * 
	 * @throws IllegalArgumentException if any argument is null
	 * @throws IOException if an I/O error occurs while working with the stream
	 * @throws SQLException if an SQL error occurs while executing the obtained definition script
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void executeDefinitions(InputStream definitionStream, Connection connection) throws IllegalArgumentException, IOException, SQLException, ParseException, TypeMappingException, RenderException {
	}
	
	/**
	 * Transforms the database definition read from a file source and executes
	 * the obtained script against a database. The JDBC connection is given as parameter.
	 * 
	 * The transformation consists of 3 phases: the parsing of the definitions from the source
	 * the rendering of the definitions to a string and the execution of the obtained output. 
	 * The format of the source input depends on the concrete DefinitionParser instance that is used
	 * (could be XSD for example). The format of the destination output depends on the concrete 
	 * DefinitionRenderer instance that is used (could be a Oracle 10g DDL producing implementation).
	 * 
	 * Implementation notes:
	 * 
	 * Call executeDefinitions(new FileInputStream(definitionFile), connection)
	 *   
	 * @param definitionFile the source definition file
	 * @param connection the connection to use
	 * 
	 * @throws IllegalArgumentException if any argument is null, string empty or all spaces
	 * @throws IOException if an I/O error occurs while working with the file
	 * @throws SQLException if an SQL error occurs while executing the obtained definition script
	 * @throws ParseException if a parser implementation specific error occurs
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void executeDefinitionsFromFile(String definitionFile, Connection connection) throws IllegalArgumentException, IOException, SQLException, ParseException, TypeMappingException, RenderException {
	}

	/**
	 * This method provides command line interface to the component.
	 * The syntax is shown below. The |, <, >, [ and ] characters have the usual
	 * significance (<> delimits a placeholder the user will have to replace with real values,
	 * [] delimits an optional switch, | is used for alternative choices).
	 * 
	 * java -jar XSD2DDL.jar -definitionFile <definitionFile> 
	 *                      [-renderFile <renderFile> | -execute [<connectionName>]]
	 *                      [-configFile <configFile>] 
	 *                      [-definitionParser <parserClass>]
	 *                      [-definitionRenderer <rendererClass>]
	 *                      
	 * -definitionFile <definitionFile>     the input definition file (usually an XSD file)
	 * -renderFile <renderFile>             the output definition file (usually a text DDL file)
	 * -execute [<connectionName>]          the output DDL is directly executed in the database connection
	 *                                      (the connection name can be explicitely given, otherwise it is taken from the config file)
	 * -configFile <configFile>             overrides the config file (that defines connection name and parser and renderer classes)
	 * -definitionParser <parserClass>      the parser class to be used (if missing it is taken from the config file)
	 * -definitionRenderer <rendererClass>  the renderer class to be used (if missing it is taken from the config file)
	 * 
	 * Note: if both -renderFile and -execute are missing then the output is written to the standard output
	 * Note: an invalid invocation will print this syntax definition
	 * 
	 * Implementation notes:
	 * 
	 * First the command line arguments are parsed
	 * 
	 * - create a CommandLineUtility instance
	 * - for each switch, create a Switch instance and add it to the CommandLineUtility instance
	 * - call parse(params) on the CommandLineUtility instance
	 * - call getValidSwitches and getInvalidSwitches to get the parsed switches and to check for invalid switches
	 * - Switch.getValue can be used on the valid switches to get the argument value
	 * 
	 * Then an instance of DefinitionUtil is created:
	 * 
	 * - the configFile (or the default config file) is loaded into the ConfigManager into the default namespace
	 * - DefinitionUtil(DEFAULT_NAMESPACE, parserClass, rendererClass) is called (null is passed for missing classes)
	 * - if renderFile is present produceDefinitionsFromFile(definitionFile, renderFile) is called
	 * - if execute is present, executeDefinitionsFromFile(definitionFile) or executeDefinitionsFromFile(definitionFile, connectionName)
	 * is called (depending on whether the connectionName argument is present)
	 * - if neither renderFile nor execute are present then produceDefinitionsFromFile(definitionFile) is called and the output
	 * string is written to the standard output
	 * 
	 * Note: any thrown exception is caught and information about the exception 
	 * (stack trace, class, message) are printed to the standard output
	 * 
	 * @param params the commmand line parameters
	 */
	public static void main(String[] params) {
	}

}
