package com.topcoder.db.generation.xsd2ddl.data;

/**
 * This is a data class that models a field type.
 * The field type has a name, a data type and 11 constraint types on the
 * domain of the type. The constraints are inspired from the XSD facets.
 * 
 * All these attributes have appropriate getters and modifiers.
 * 
 * The purpose of this class (and of the entire data class structure) is to
 * provide definition data exchange between the parsing and rendering layers
 * in a format that is independent of the concrete representation details (XSD, DDL).
 * 
 * Thread safety:
 * 
 * This is a mutable data class so it is not thread safe.
 * However, the class is used in such a way that two threads do not share an instance
 * so the class is used in a thread safe way.
 * 
 * @author TCSDESIGNER
 */
public class FieldType {
	
	/**
	 * The name of the type.
	 * Can be null (in case of types directly nested in XSD) but cannot be empty or all spaces.
	 * Used by its setter and getter. 
	 */
	private String typeName = null;

	/**
	 * The data type.
	 * Cannot be null, empty or all spaces (but it is null until set first).
	 * Used by its setter and getter. 
	 */
	private String dataType = null;
	
	/**
	 * The enumeration values.
	 * Can be null, but cannot contain nulls or have zero elements.
	 * Used by its setter and getter. 
	 */
	private String[] enumeration = null;

	/**
	 * The number of fraction digits.
	 * Can be either -1 (meaning it is not set) or a positive number (including 0).
	 * Used by its setter and getter. 
	 */
	private int fractionDigits = -1;

	/**
	 * The length.
	 * Can be either -1 (meaning it is not set) or a positive number (excluding 0).
	 * Used by its setter and getter. 
	 */
    private int length = -1;
    
	/**
	 * The maximum exclusive value.
	 * Can be null but not empty or all spaces.
	 * Used by its setter and getter. 
	 */
	private String maxExclusive = null;
	
	/**
	 * The maximum inclusiv value.
	 * Can be null but not empty or all spaces.
	 * Used by its setter and getter. 
	 */
	private String maxInclusive = null;
	
	/**
	 * The maximum length.
	 * Can be either -1 (meaning it is not set) or a positive number (excluding 0).
	 * Used by its setter and getter. 
	 */
	private int maxLength = -1;

	/**
	 * The minimum exclusive value.
	 * Can be null but not empty or all spaces.
	 * Used by its setter and getter. 
	 */
    private String minExclusive = null;

    /**
	 * The minimum inclusive value.
	 * Can be null but not empty or all spaces.
	 * Used by its setter and getter. 
	 */
	private String minInclusive = null;
	
	/**
	 * The minimum length.
	 * Can be either -1 (meaning it is not set) or a positive number (excluding 0).
	 * Used by its setter and getter. 
	 */
	private int minLength = -1;
	
	/**
	 * The regular expression pattern.
	 * Can be null, but not empty or all spaces.
	 * Used by its setter and getter. 
	 */
	private String pattern = null;
	
	/**
	 * The total number of digits.
	 * Can be either -1 (meaning it is not set) or a positive number (excluding 0).
	 * Used by its setter and getter. 
	 */
	private int totalDigits = -1;
	
    /**
     * Default constructor.
     * Doesn't do anything.
     */
	public FieldType() {
	}

	/**
	 * Gets the type name (typeName field)
	 * 
	 * @return the type name
	 */
	public String getTypeName() {
		return typeName;
	}

	/**
	 * Sets the type name (typeName field)
	 * 
	 * @param typeName the type name to set (can be null)
	 * 
	 * @throws IllegalArgumentException if typeName is empty or all spaces
	 */
	public void setTypeName(String typeName) throws IllegalArgumentException {
		this.typeName = typeName;
	}

	/**
	 * Gets the data type (dataType field)
	 * 
	 * @return the data type
	 */
	public String getDataType() {
		return dataType;
	}

	/**
	 * Sets the data type (dataType field)
	 * 
	 * @param dataType the data type to set
	 * 
	 * @throws IllegalArgumentException if argument is null
	 */
	public void setDataType(String dataType) throws IllegalArgumentException {
		this.dataType = dataType;
	}

	/**
	 * Gets the enumeration values (enumeration field - no copy)
	 * 
	 * @return the enumeration values
	 */
	public String[] getEnumeration() {
		return enumeration;
	}

	/**
	 * Sets the enumeration values (enumeration field - no copy)
	 * 
	 * @param enumeration the enumeration values to set (can be null)
	 * 
	 * @throws IllegalArgumentException if enumeration has size 0 or contains nulls
	 */
	public void setEnumeration(String[] enumeration) throws IllegalArgumentException {
		this.enumeration = enumeration;
	}

	/**
	 * Gets the fraction digits (fractionDigits field)
	 * 
	 * @return the fraction digits
	 */
	public int getFractionDigits() {
		return fractionDigits;
	}

	/**
	 * Sets the fraction digits (fractionDigits field)
	 * 
	 * @param fractionDigits the fraction digits to set
	 * 
	 * @throws IllegalArgumentException if argument <0 (other than -1)
	 */
	public void setFractionDigits(int fractionDigits) throws IllegalArgumentException {
		this.fractionDigits = fractionDigits;
	}

	/**
	 * Gets the length (length field)
	 * 
	 * @return the length
	 */
	public int getLength() {
		return length;
	}

	/**
	 * Sets the length (length field)
	 * 
	 * @param length the length to set
	 * 
	 * @throws IllegalArgumentException if argument <=0 (other than -1)
	 */
	public void setLength(int length) throws IllegalArgumentException {
		this.length = length;
	}

	/**
	 * Gets the maximum exclusive limit (maxExclusive field)
	 * 
	 * @return the maximum exclusive limit
	 */
	public String getMaxExclusive() {
		return maxExclusive;
	}

	/**
	 * Sets the maximum exclusive limit (maxExclusive field)
	 * 
	 * @param maxExclusive the maximum exclusive limit to set (can be null)
	 * 
	 * @throws IllegalArgumentException if argument empty or all spaces
	 */
	public void setMaxExclusive(String maxExclusive) throws IllegalArgumentException {
		this.maxExclusive = maxExclusive;
	}

	/**
	 * Gets the maximum inclusive limit (maxInclusive field)
	 * 
	 * @return the maximum inclusive limit
	 */
	public String getMaxInclusive() {
		return maxInclusive;
	}

	/**
	 * Sets the maximum inclusive limit (maxInclusive field)
	 * 
	 * @param maxInclusive the maximum inclusive limit to set (can be null)
	 * 
	 * @throws IllegalArgumentException if argument empty or all spaces
	 */
	public void setMaxInclusive(String maxInclusive) throws IllegalArgumentException {
		this.maxInclusive = maxInclusive;
	}

	/**
	 * Gets the maximum length (maxLength field)
	 * 
	 * @return the maximum length
	 */
	public int getMaxLength() {
		return maxLength;
	}

	/**
	 * Sets the maximum length (maxLength field)
	 * 
	 * @param maxLength the maximum length to set
	 * 
	 * @throws IllegalArgumentException if argument <=0 (other than -1)
	 */
	public void setMaxLength(int maxLength) throws IllegalArgumentException {
		this.maxLength = maxLength;
	}

	/**
	 * Gets the minimum exclusive value (minExclusive field)
	 * 
	 * @return the minimum exclusive value
	 */
	public String getMinExclusive() {
		return minExclusive;
	}

	/**
	 * Sets the minimum exclusive value (minExclusive field)
	 * 
	 * @param minExclusive the minimum exclusive value to set (can be null)
	 * 
	 * @throws IllegalArgumentException if argument empty or all spaces
	 */
	public void setMinExclusive(String minExclusive) throws IllegalArgumentException {
		this.minExclusive = minExclusive;
	}

	/**
	 * Gets the minimum inclusive value (minInclusive field)
	 * 
	 * @return the minimum inclusive value
	 */
	public String getMinInclusive() {
		return minInclusive;
	}

	/**
	 * Sets the minimum inclusive value (minInclusive field)
	 * 
	 * @param minInclusive the minimum inclusive value to set (can be null)
	 * 
	 * @throws IllegalArgumentException if argument empty or all spaces
	 */
	public void setMinInclusive(String minInclusive) throws IllegalArgumentException {
		this.minInclusive = minInclusive;
	}

	/**
	 * Gets the minimum length (minLengh field)
	 * 
	 * @return the minimum length
	 */
	public int getMinLength() {
		return minLength;
	}

	/**
	 * Sets the minimum length (minLengh field)
	 * 
	 * @param minLength the minimum length to set
	 * 
	 * @throws IllegalArgumentException if argument <=0 (other than -1)
	 */
	public void setMinLength(int minLength) throws IllegalArgumentException {
		this.minLength = minLength;
	}

	/**
	 * Gets the pattern (pattern field)
	 * 
	 * @return the pattern
	 */
	public String getPattern() {
		return pattern;
	}

	/**
	 * Sets the pattern (pattern field)
	 * 
	 * @param pattern the pattern to set (can be null)
	 * 
	 * @throws IllegalArgumentException if argument empty or all spaces
	 */
	public void setPattern(String pattern) throws IllegalArgumentException {
		this.pattern = pattern;
	}

	/**
	 * Gets the total digits (totalDigits field)
	 * 
	 * @return the total digits
	 */
	public int getTotalDigits() {
		return totalDigits;
	}

	/**
	 * Sets the total digits (totalDigits field)
	 * 
	 * @param totalDigits the total digits to set
	 * 
	 * @throws IllegalArgumentException if argument <=0 (other than -1)
	 */
	public void setTotalDigits(int totalDigits) throws IllegalArgumentException {
		this.totalDigits = totalDigits;
	}

}
