package com.topcoder.db.generation.xsd2ddl.mappings;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.topcoder.db.generation.xsd2ddl.TypeMapping;
import com.topcoder.db.generation.xsd2ddl.TypeMappingException;
import com.topcoder.db.generation.xsd2ddl.data.FieldType;
import com.topcoder.util.config.ConfigManagerException;

/**
 * This is the default implemenation of the TypeMapping interface.
 * It implements type mapping between a definition source and a target based on the
 * information put in a configuration file.
 * 
 * This class takes a FieldType object as input and decides based on some configuration
 * rules what would be the type to represent the source type in the target
 * type system.
 *
 * The configuration rules specify the source types (there can be more), the target
 * type and whether the type has a length (so that the length can be appended enclosed
 * in paranthesis), a number of digits or decimals.
 * 
 * Note:
 * 
 * The implementation has a public constructor taking a configuration namespace (String)
 * as parameter. This constructor is invoked by the DefinitionRenderer implementations that 
 * need such type mapping service. This allows the mapping implementation to read the rules
 * from a configuration namespace.
 *
 * Thread safety:
 * 
 * The map and sets are populated in the constructor. The query method simply consult them
 * so the class is thread safe since it is imutable.
 * 
 * @author TCSDESIGNER
 */
public class ConfigMapping implements TypeMapping {
	
	/**
	 * The type mapping.
	 * Maps source types (String) to target types (String).
	 * Cannot have nulls, empty or all spaces strings as keys or elements.
	 * Set in the constructor, used by the method.
	 */
	private Map typeMapping = new HashMap();
	
	/**
	 * The set of target types having length.
	 * Contains target types (String).
	 * Cannot contain nulls, empty or all spaces strings.
	 * Set in the constructor, used by the method.
	 */
	private Set typesWithLength = new HashSet();

	/**
	 * The set of target types having total digits.
	 * Contains target types (String).
	 * Cannot contain nulls, empty or all spaces strings.
	 * Set in the constructor, used by the method.
	 */
    private Set typesWithDigits = new HashSet();
	
	/**
	 * The set of target types having decimals.
	 * Contains target types (String).
	 * Cannot contain nulls, empty or all spaces strings.
	 * Set in the constructor, used by the method.
	 */
     private Set typesWithDecimals = new HashSet();
	
	/**
	 * Constructor taking the namespace as parameters.
	 * It reads the type mapping configuration data.
	 * 
	 * Implementation details:
	 * 
	 * - reads the config_type_mapping property
	 * - iterates all its nested properties (each one is a mapping rule)
	 * - for each mapping rule read from_types, to_type, has_length, has_digits, has_decimals.
	 * - for each s in from_types, add a (s,to_type) entry in the type mapping
	 * - if has_length present add target type to typesWithLength
	 * - if has_digits present add target type to typesWithDigits
	 * - if has_decimals present add target type to typesWithDecimals
	 * 
	 * @param namespace the configuration namespace
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all space
	 * @throws ConfigManagerException if any configuration exception occurs or a required field is missing
	 */
	public ConfigMapping(String namespace) throws IllegalArgumentException, ConfigManagerException {
	}

	/**
	 * Converts a source field type to a target type. 
	 * The target type is a string suitable in SQL statements for example.
	 *  
	 * Implementation details:
	 * 
	 * - get corresponding type for type.getTypeName() from the type mapping
	 * - set s = corresponding type 
	 * - if the corresponding type has length (contained in typesWithLength) append "(" + type.getLength() + ")"
	 * - if the corresponding type has digits (contained in typesWithDigits) append "(" + type.getTotalDigits() + ")"
	 * - if the corresponding type has decimals too (contained in typesWithDecimals) 
	 *   append "(" + type.getTotalDigits() + "," + type.getFractionDigits() + ")"
	 * 
	 * @param type the field type the mapped type is obtained for
	 * @return the type that can be used in the destination system
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws TypeMappingException if a type mapping implementation specific error occurs
	 */
	public String convertType(FieldType type) throws IllegalArgumentException, TypeMappingException {
		return null;
	}

}
